@@ -2,15 +2,32 @@ import abc
 import collections
 import dataclasses
 import datetime
+import decimal
+import inspect
 import json
+import random
+import sys
 import typing
 import uuid
 from collections import OrderedDict
 
 import inflect
+from faker import Faker
+from pytz import utc
 
-from dataclasses_avroschema import types, utils
+from dataclasses_avroschema import schema_generator, serialization, types, utils
 
+from .exceptions import NameSpaceRequiredException
+
+PY_VER = sys.version_info
+
+if PY_VER >= (3, 9):
+    GenericAlias = (typing.GenericAlias, typing._GenericAlias, typing._SpecialGenericAlias, typing._UnionGenericAlias)
+else:
+    GenericAlias = typing._GenericAlias
+
+
+fake = Faker()
 p = inflect.engine()
 
 BOOLEAN = "boolean"
@@ -18,6 +35,7 @@ NULL = "null"
 INT = "int"
 FLOAT = "float"
 LONG = "long"
+DOUBLE = "double"
 BYTES = "bytes"
 STRING = "string"
 ARRAY = "array"
@@ -28,6 +46,7 @@ DATE = "date"
 TIME_MILLIS = "time-millis"
 TIMESTAMP_MILLIS = "timestamp-millis"
 UUID = "uuid"
+DECIMAL = "decimal"
 LOGICAL_DATE = {"type": INT, "logicalType": DATE}
 LOGICAL_TIME = {"type": INT, "logicalType": TIME_MILLIS}
 LOGICAL_DATETIME = {"type": LONG, "logicalType": TIMESTAMP_MILLIS}
@@ -36,8 +55,8 @@ LOGICAL_UUID = {"type": STRING, "logicalType": UUID}
 PYTHON_TYPE_TO_AVRO = {
     bool: BOOLEAN,
     type(None): NULL,
-    int: INT,
-    float: FLOAT,
+    int: LONG,
+    float: DOUBLE,
     bytes: BYTES,
     str: STRING,
     list: {"type": ARRAY},
@@ -45,6 +64,8 @@ PYTHON_TYPE_TO_AVRO = {
     dict: {"type": MAP},
     types.Fixed: {"type": FIXED},
     types.Enum: {"type": ENUM},
+    types.Int32: INT,
+    types.Float32: FLOAT,
     datetime.date: {"type": INT, "logicalType": DATE},
     datetime.time: {"type": INT, "logicalType": TIME_MILLIS},
     datetime.datetime: {"type": LONG, "logicalType": TIMESTAMP_MILLIS},
@@ -52,35 +73,47 @@ PYTHON_TYPE_TO_AVRO = {
 }
 
 # excluding tuple because is a container
-PYTHON_INMUTABLE_TYPES = (str, int, bool, float, bytes, type(None))
+PYTHON_INMUTABLE_TYPES = (str, int, types.Int32, types.Float32, bool, float, bytes, type(None))
 
 PYTHON_PRIMITIVE_CONTAINERS = (list, tuple, dict)
 
-PYTHON_LOGICAL_TYPES = (
-    datetime.date,
-    datetime.time,
-    datetime.datetime,
-    uuid.uuid4,
-    uuid.UUID,
-)
+PYTHON_LOGICAL_TYPES = (datetime.date, datetime.time, datetime.datetime, uuid.uuid4, uuid.UUID, decimal.Decimal)
 
 PYTHON_PRIMITIVE_TYPES = PYTHON_INMUTABLE_TYPES + PYTHON_PRIMITIVE_CONTAINERS
 
 PRIMITIVE_AND_LOGICAL_TYPES = PYTHON_INMUTABLE_TYPES + PYTHON_LOGICAL_TYPES
 
 PythonImnutableTypes = typing.Union[
-    str, int, bool, float, list, tuple, dict, datetime.date, datetime.time, datetime.datetime, uuid.UUID
+    str,
+    int,
+    types.Int32,
+    bool,
+    float,
+    types.Float32,
+    list,
+    tuple,
+    dict,
+    datetime.date,
+    datetime.time,
+    datetime.datetime,
+    uuid.UUID,
+    decimal.Decimal,
 ]
 
 
 @dataclasses.dataclass  # type: ignore
 class BaseField:
-    avro_type: typing.ClassVar
+    avro_type: typing.ClassVar[str]
 
     name: str
     type: typing.Any  # store the python primitive type
     default: typing.Any
+    parent: typing.Tuple[dataclasses.Field]
     metadata: typing.Mapping = dataclasses.field(default_factory=dict)
+    model_metadata: typing.Optional[utils.SchemaMetadata] = None
+
+    def __post_init__(self) -> None:
+        self.model_metadata = self.model_metadata or utils.SchemaMetadata()
 
     @staticmethod
     def _get_self_reference_type(a_type: typing.Any) -> str:
@@ -143,7 +176,10 @@ class BaseField:
 
     def validate_default(self) -> bool:
         msg = f"Invalid default type. Default should be {self.type}"
-        assert isinstance(self.default, self.type), msg
+        if getattr(self.type, "__metadata__", [None])[0] in types.CUSTOM_TYPES:
+            assert isinstance(self.default, self.type.__origin__)
+        else:
+            assert isinstance(self.default, self.type), msg
 
         return True
 
@@ -157,8 +193,11 @@ class BaseField:
     def get_avro_type(self) -> typing.Any:
         ...  # pragma: no cover
 
+    def fake(self) -> typing.Any:
+        return None
 
-class InmutableField(BaseField):
+
+class ImmutableField(BaseField):
     def get_avro_type(self) -> PythonImnutableTypes:
         if self.default is None:
             return [NULL, self.avro_type]
@@ -166,28 +205,56 @@ class InmutableField(BaseField):
 
 
 @dataclasses.dataclass
-class StringField(InmutableField):
-    avro_type: typing.ClassVar = STRING
+class StringField(ImmutableField):
+    avro_type: typing.ClassVar[str] = STRING
+
+    def fake(self) -> str:
+        return fake.pystr()
 
 
 @dataclasses.dataclass
-class IntegerField(InmutableField):
-    avro_type: typing.ClassVar = INT
+class IntField(ImmutableField):
+    avro_type: typing.ClassVar[str] = INT
+
+    def fake(self) -> int:
+        return fake.pyint()
 
 
 @dataclasses.dataclass
-class BooleanField(InmutableField):
-    avro_type: typing.ClassVar = BOOLEAN
+class LongField(ImmutableField):
+    avro_type: typing.ClassVar[str] = LONG
+
+    def fake(self) -> int:
+        return fake.pyint()
 
 
 @dataclasses.dataclass
-class FloatField(InmutableField):
-    avro_type: typing.ClassVar = FLOAT
+class BooleanField(ImmutableField):
+    avro_type: typing.ClassVar[str] = BOOLEAN
+
+    def fake(self) -> bool:
+        return fake.pybool()
 
 
 @dataclasses.dataclass
-class BytesField(InmutableField):
-    avro_type: typing.ClassVar = BYTES
+class DoubleField(ImmutableField):
+    avro_type: typing.ClassVar[str] = DOUBLE
+
+    def fake(self) -> float:
+        return fake.pyfloat()
+
+
+@dataclasses.dataclass
+class FloatField(ImmutableField):
+    avro_type: typing.ClassVar[str] = FLOAT
+
+    def fake(self) -> float:
+        return fake.pyfloat()  # Roughly the range on a float32
+
+
+@dataclasses.dataclass
+class BytesField(ImmutableField):
+    avro_type: typing.ClassVar[str] = BYTES
 
     def get_default_value(self) -> typing.Any:
         if self.default in (dataclasses.MISSING, None):
@@ -200,10 +267,13 @@ class BytesField(InmutableField):
     def to_avro(item: bytes) -> str:
         return item.decode()
 
+    def fake(self) -> bytes:
+        return fake.pystr().encode()
+
 
 @dataclasses.dataclass
-class NoneField(InmutableField):
-    avro_type: typing.ClassVar = NULL
+class NoneField(ImmutableField):
+    avro_type: typing.ClassVar[str] = NULL
 
 
 @dataclasses.dataclass
@@ -220,12 +290,11 @@ class ContainerField(BaseField):
 @dataclasses.dataclass
 class ListField(ContainerField):
     items_type: typing.Any = None
-
-    def __post_init__(self) -> None:
-        self.generate_items_type()
+    internal_field: typing.Any = None
 
     @property
     def avro_type(self) -> typing.Dict:
+        self.generate_items_type()
         return {"type": ARRAY, "items": self.items_type}
 
     def get_default_value(self) -> typing.Union[typing.List, dataclasses._MISSING_TYPE]:
@@ -254,26 +323,32 @@ class ListField(ContainerField):
         name = self.get_singular_name(self.name)
 
         if utils.is_union(items_type):
-            self.items_type = UnionField.generate_union(
+            self.items_type = UnionField(
                 name,
-                items_type.__args__,
+                items_type,
                 default=self.default,
                 default_factory=self.default_factory,
-            )
+                model_metadata=self.model_metadata,
+                parent=self.parent,
+            ).get_avro_type()
         else:
-            field = AvroField(name, items_type)
-            self.items_type = field.get_avro_type()
+            self.internal_field = AvroField(name, items_type, model_metadata=self.model_metadata, parent=self.parent)
+            self.items_type = self.internal_field.get_avro_type()
+
+    def fake(self) -> typing.List:
+        self.generate_items_type()
+        # return a list of one element with the type specified
+        return [self.internal_field.fake()]
 
 
 @dataclasses.dataclass
 class DictField(ContainerField):
     values_type: typing.Any = None
-
-    def __post_init__(self) -> None:
-        self.generate_values_type()
+    internal_field: typing.Any = None
 
     @property
     def avro_type(self) -> typing.Dict[str, typing.Any]:
+        self.generate_values_type()
         return {"type": MAP, "values": self.values_type}
 
     def get_default_value(self) -> typing.Union[typing.Dict[str, typing.Any], dataclasses._MISSING_TYPE]:
@@ -304,27 +379,22 @@ class DictField(ContainerField):
         values_type = self.type.__args__[1]
 
         name = self.get_singular_name(self.name)
-        field = AvroField(name, values_type)
-        self.values_type = field.get_avro_type()
+        self.internal_field = AvroField(name, values_type, model_metadata=self.model_metadata, parent=self.parent)
+        self.values_type = self.internal_field.get_avro_type()
+
+    def fake(self) -> typing.Dict[str, typing.Any]:
+        # return a dict of one element with the items type specified
+        self.generate_values_type()
+        return {fake.pystr(): self.internal_field.fake()}
 
 
 @dataclasses.dataclass
 class UnionField(BaseField):
     default_factory: typing.Optional[typing.Callable] = None
+    unions: typing.List = dataclasses.field(default_factory=list)
+    internal_fields: typing.List = dataclasses.field(default_factory=list)
 
-    def get_avro_type(self) -> typing.List:
-        elements = self.type.__args__
-        # generate the singular name in the case that we have array or map
-        name = self.get_singular_name(self.name)
-        return self.generate_union(name, elements, default=self.default, default_factory=self.default_factory)
-
-    @staticmethod
-    def generate_union(
-        name: str,
-        elements: typing.List,
-        default: typing.Optional[typing.Any] = None,
-        default_factory: typing.Optional[typing.Callable] = None,
-    ) -> typing.List:
+    def generate_unions_type(self) -> typing.List:
         """
         Generate union.
 
@@ -337,24 +407,43 @@ class UnionField(BaseField):
         Returns:
             typing.List: List of avro types
         """
-        unions = []
+        elements = self.type.__args__
+        name = self.get_singular_name(self.name)
+
+        unions: typing.List = []
+
+        # Place default at front of list
+        default_type = None
+        if self.default is None and self.default_factory is dataclasses.MISSING:
+            unions.insert(0, NULL)
+        elif type(self.default) is not dataclasses._MISSING_TYPE:
+            default_type = type(self.default)
+            default_field = AvroField(name, default_type, model_metadata=self.model_metadata, parent=self.parent)
+            unions.append(default_field.get_avro_type())
+            self.internal_fields.append(default_field)
+
         for element in elements:
             # create the field and get the avro type
-            field = AvroField(name, element)
-            unions.append(field.get_avro_type())
+            field = AvroField(name, element, model_metadata=self.model_metadata, parent=self.parent)
+            avro_type = field.get_avro_type()
 
-        if default is None and default_factory is dataclasses.MISSING and NULL not in unions:
-            unions.insert(0, NULL)
+            if avro_type not in unions:
+                unions.append(avro_type)
+                self.internal_fields.append(field)
 
         return unions
 
+    def get_avro_type(self) -> typing.List:
+        self.unions = self.generate_unions_type()
+        return self.unions
+
     def get_default_value(self) -> typing.Any:
         is_default_factory_callable = callable(self.default_factory)
 
         if self.default in (dataclasses.MISSING, None) and not is_default_factory_callable:
             return self.default
         elif is_default_factory_callable:
-            # expeting a callable
+            # expecting a callable
             default = self.default_factory()  # type: ignore
             assert isinstance(default, (dict, list)), f"Dict or List is required as default for field {self.name}"
 
@@ -363,6 +452,12 @@ class UnionField(BaseField):
             return LOGICAL_TYPES_FIELDS_CLASSES[type(self.default)].to_avro(self.default)  # type: ignore
         return self.default
 
+    def fake(self) -> typing.Any:
+        self.unions = self.generate_unions_type()
+        # get a random internal field and return a fake value
+        field = random.choice(self.internal_fields)
+        return field.fake()
+
 
 @dataclasses.dataclass
 class FixedField(BaseField):
@@ -384,6 +479,9 @@ class FixedField(BaseField):
     def get_default_value(self) -> dataclasses._MISSING_TYPE:
         return dataclasses.MISSING
 
+    def fake(self) -> bytes:
+        return fake.pystr(max_chars=self.default.size).encode()
+
 
 @dataclasses.dataclass
 class EnumField(BaseField):
@@ -416,6 +514,9 @@ class EnumField(BaseField):
             ), f"The default value should be on of {self.default.symbols}. Current is {default}"
             return default
 
+    def fake(self) -> typing.Any:
+        return random.choice(self.default.symbols)
+
 
 @dataclasses.dataclass
 class SelfReferenceField(BaseField):
@@ -434,7 +535,7 @@ class SelfReferenceField(BaseField):
         return dataclasses.MISSING
 
 
-class LogicalTypeField(InmutableField):
+class LogicalTypeField(ImmutableField):
     def get_default_value(self) -> typing.Union[None, str, int, float]:
         if self.default in (dataclasses.MISSING, None):
             return self.default
@@ -458,7 +559,7 @@ class DateField(LogicalTypeField):
     the number of days from the unix epoch, 1 January 1970 (ISO calendar).
     """
 
-    avro_type: typing.ClassVar = LOGICAL_DATE
+    avro_type: typing.ClassVar[str] = LOGICAL_DATE
 
     @staticmethod
     def to_avro(date: datetime.date) -> int:
@@ -478,6 +579,9 @@ class DateField(LogicalTypeField):
 
         return int(ts / (3600 * 24))
 
+    def fake(self) -> datetime.date:
+        return fake.date_object()
+
 
 @dataclasses.dataclass
 class TimeField(LogicalTypeField):
@@ -490,7 +594,7 @@ class TimeField(LogicalTypeField):
     where the int stores the number of milliseconds after midnight, 00:00:00.000.
     """
 
-    avro_type: typing.ClassVar = LOGICAL_TIME
+    avro_type: typing.ClassVar[str] = LOGICAL_TIME
 
     @staticmethod
     def to_avro(time: datetime.time) -> int:
@@ -513,6 +617,9 @@ class TimeField(LogicalTypeField):
 
         return int((((hour * 60 + minutes) * 60 + seconds) * 1000) + (microseconds / 1000))
 
+    def fake(self) -> datetime.time:
+        return fake.time_object()
+
 
 @dataclasses.dataclass
 class DatetimeField(LogicalTypeField):
@@ -525,7 +632,7 @@ class DatetimeField(LogicalTypeField):
     1 January 1970 00:00:00.000 UTC.
     """
 
-    avro_type: typing.ClassVar = LOGICAL_DATETIME
+    avro_type: typing.ClassVar[str] = LOGICAL_DATETIME
 
     @staticmethod
     def to_avro(date_time: datetime.datetime) -> float:
@@ -546,10 +653,13 @@ class DatetimeField(LogicalTypeField):
 
         return ts * 1000
 
+    def fake(self) -> datetime.datetime:
+        return fake.date_time(tzinfo=utc)
+
 
 @dataclasses.dataclass
 class UUIDField(LogicalTypeField):
-    avro_type: typing.ClassVar = LOGICAL_UUID
+    avro_type: typing.ClassVar[str] = LOGICAL_UUID
 
     def validate_default(self) -> bool:
         msg = f"Invalid default type. Default should be {str} or {uuid.UUID}"
@@ -561,21 +671,121 @@ class UUIDField(LogicalTypeField):
     def to_avro(uuid: uuid.UUID) -> str:
         return str(uuid)
 
+    def fake(self) -> uuid.UUID:
+        return uuid.uuid4()
+
 
 @dataclasses.dataclass
 class RecordField(BaseField):
     def get_avro_type(self) -> typing.Union[typing.List, typing.Dict]:
-        record_type = self.type.avro_schema_to_python()
+        alias = self.model_metadata.get_alias(self.name)  # type: ignore
+        name = alias or self.type.__name__
+
+        if not self.exist_type():
+            user_defined_type = utils.UserDefinedType(name=name, type=self.type)
+            self.parent.user_defined_types = self.parent.user_defined_types + (user_defined_type,)
+
+            record_type = self.type.avro_schema_to_python()
+            record_type["name"] = name
+        else:
+            meta = getattr(self.type, "Meta", None)
+            metadata = utils.SchemaMetadata.create(meta)
+
+            if metadata.namespace is None:
+                raise NameSpaceRequiredException(field_type=self.type, field_name=self.name)
+            record_type = f"{metadata.namespace}.{name}"
 
         if self.default is None:
             return [NULL, record_type]
         return record_type
 
+    def fake(self) -> typing.Any:
+        return self.type.fake()
+
+    def exist_type(self):
+        # filter by the same field types
+        same_types = [
+            field.type
+            for field in self.parent.user_defined_types
+            if field.type == self.type and field.name != self.name
+        ]
+
+        # If length > 0, means that it is the first appearance
+        # of this type, otherwise exist already.
+        return len(same_types)
+
+
+@dataclasses.dataclass
+class DecimalField(BaseField):
+
+    precision: int = -1
+    scale: int = 0
+
+    def __post_init__(self) -> None:
+        self.set_precision_scale()
+
+    def set_precision_scale(self) -> None:
+        if self.default != types.MissingSentinel:
+            if isinstance(self.default, decimal.Decimal):
+                sign, digits, scale = self.default.as_tuple()
+                self.scale = scale * -1  # Make scale positive, as that's what Avro expects
+                # decimal.Context has a precision property
+                # BUT the precision property is independent of the number of digits stored in the Decimal instance
+                # # # FROM THE DOCS HERE https://docs.python.org/3/library/decimal.html
+                #  The context precision does not affect how many digits are stored.
+                #  That is determined exclusively by the number of digits in value.
+                #  For example, Decimal('3.00000') records all five zeros even if the context precision is only three.
+                # # #
+                # Avro is concerned with *what form the number takes* and not with *handling errors in the Python env*
+                # so we take the number of digits stored in the decimal as Avro precision
+                self.precision = len(digits)
+            elif isinstance(self.default, types.Decimal):
+                self.scale = self.default.scale
+                self.precision = self.default.precision
+            else:
+                raise ValueError("decimal.Decimal default types must be either decimal.Decimal or types.Decimal")
+        else:
+            raise ValueError(
+                "decimal.Decimal default types must be specified to provide precision and scale,"
+                " and must be either decimal.Decimal or types.Decimal"
+            )
+
+        # Validation on precision and scale per Avro schema
+        if self.precision <= 0:
+            raise ValueError("Precision must be a positive integer greater than zero")
+
+        if self.scale < 0 or self.precision < self.scale:
+            raise ValueError("Scale must be zero or a positive integer less than or equal to the precision.")
+
+            # Just pull the precision from default context and default out scale
+            # Not ideal
+            #
+            # self.precision = decimal.Context().prec
+
+    def get_avro_type(self) -> typing.Dict[str, typing.Any]:
+        avro_type = {"type": BYTES, "logicalType": DECIMAL, "precision": self.precision, "scale": self.scale}
+
+        return avro_type
+
+    def get_default_value(self) -> typing.Union[str, dataclasses._MISSING_TYPE, None]:
+        default = self.default
+        if isinstance(default, types.Decimal):
+            default = default.default
+
+        if default == types.MissingSentinel:
+            return dataclasses.MISSING
+        return serialization.decimal_to_str(default, self.precision, self.scale)
+
+    def fake(self) -> decimal.Decimal:
+        return fake.pydecimal(right_digits=self.scale, left_digits=self.precision - self.scale)
+
 
 INMUTABLE_FIELDS_CLASSES = {
     bool: BooleanField,
-    int: IntegerField,
-    float: FloatField,
+    int: LongField,
+    types.Int32: IntField,
+    float: DoubleField,
+    types.Float32: FloatField,
     bytes: BytesField,
     str: StringField,
     type(None): NoneField,
@@ -599,6 +809,7 @@ LOGICAL_TYPES_FIELDS_CLASSES = {
     uuid.uuid4: UUIDField,
     uuid.UUID: UUIDField,
     bytes: BytesField,
+    decimal.Decimal: DecimalField,
 }
 
 PRIMITIVE_LOGICAL_TYPES_FIELDS_CLASSES = {
@@ -612,8 +823,9 @@ LOGICAL_CLASSES = LOGICAL_TYPES_FIELDS_CLASSES.keys()
 
 FieldType = typing.Union[
     StringField,
+    LongField,
     BooleanField,
-    FloatField,
+    DoubleField,
     BytesField,
     NoneField,
     ListField,
@@ -627,27 +839,58 @@ FieldType = typing.Union[
     DatetimeField,
     UUIDField,
     RecordField,
-    InmutableField,
+    ImmutableField,
 ]
 
 
 def field_factory(
     name: str,
     native_type: typing.Any,
+    parent: typing.Any = None,
+    *,
     default: typing.Any = dataclasses.MISSING,
     default_factory: typing.Any = dataclasses.MISSING,
     metadata: typing.Mapping = dataclasses.field(default_factory=dict),
+    model_metadata: typing.Optional[utils.SchemaMetadata] = None,
 ) -> FieldType:
     if native_type in PYTHON_INMUTABLE_TYPES:
         klass = INMUTABLE_FIELDS_CLASSES[native_type]
-        return klass(name=name, type=native_type, default=default, metadata=metadata)
+        return klass(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     elif utils.is_self_referenced(native_type):
-        return SelfReferenceField(name=name, type=native_type, default=default, metadata=metadata)
+        return SelfReferenceField(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     elif native_type is types.Fixed:
-        return FixedField(name=name, type=native_type, default=default, metadata=metadata)
+        return FixedField(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     elif native_type is types.Enum:
-        return EnumField(name=name, type=native_type, default=default, metadata=metadata)
-    elif isinstance(native_type, typing._GenericAlias):  # type: ignore
+        return EnumField(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
+    elif isinstance(native_type, GenericAlias):  # type: ignore
         origin = native_type.__origin__
 
         if origin not in (
@@ -667,18 +910,41 @@ def field_factory(
             )
 
         container_klass = CONTAINER_FIELDS_CLASSES[origin]
-        return container_klass(
+        return container_klass(  # type: ignore
             name=name,
             type=native_type,
             default=default,
             metadata=metadata,
             default_factory=default_factory,
+            model_metadata=model_metadata,
+            parent=parent,
         )
     elif native_type in PYTHON_LOGICAL_TYPES:
-        klass = LOGICAL_TYPES_FIELDS_CLASSES[native_type]
-        return klass(name=name, type=native_type, default=default, metadata=metadata)
+        klass = LOGICAL_TYPES_FIELDS_CLASSES[native_type]  # type: ignore
+        return klass(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
+    elif inspect.isclass(native_type) and issubclass(native_type, schema_generator.AvroModel):
+        return RecordField(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     else:
-        return RecordField(name=name, type=native_type, default=default, metadata=metadata)
+        msg = (
+            f"Type {native_type} is unknown. Please check the valid types at "
+            "https://marcosschroh.github.io/dataclasses-avroschema/fields_specification/#avro-field-and-python-types-summary"  # noqa: E501
+        )
+
+        raise ValueError(msg)
 
 
 AvroField = field_factory
